@@ -13,7 +13,7 @@
 ;;;  warranty.  In no circumstances the author(s) shall be liable
 ;;;  for any damages arising out of the use of this software.
 ;;;
-;;; $Id: matrix.scm,v 1.2 2004/04/02 01:13:53 foof Exp $
+;;; $Id: matrix.scm,v 1.4 2006/04/06 04:09:31 shirok Exp $
 ;;;
 
 (select-module gauche.array)
@@ -236,6 +236,7 @@
            (subarray tmp (shape (s32vector-ref start 0) (s32vector-ref end 0)
                                 (s32vector-ref end 1) (+ (s32vector-ref end 1) n)))))))
 
+
 (define (determinant! a)
   (let* ((start (s32vector->list (start-vector-of a)))
          (end (s32vector->list (end-vector-of a)))
@@ -255,10 +256,10 @@
             (eq? class <f64array>)
             (eq? class <array>))
       (determinant! (copy-object a))
-      (let ((rank (s32vector-length (start-vector-of a)))
-            (b (tabulate-array (array-shape a)
-                 (lambda (ind) (array-ref a ind))
-                 (make-vector rank))))
+      (let* ((rank (s32vector-length (start-vector-of a)))
+             (b (tabulate-array (array-shape a)
+                                (lambda (ind) (array-ref a ind))
+                                (make-vector rank))))
         (determinant! b)))))
 
 
@@ -594,3 +595,6 @@
         (if (eq? next <array>)
           (return <array>) ; already most general array
           (loop next (cdr l)))))))
+
+(provide "gauche/matrix")
+
