@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: vport.c,v 1.10 2004/11/20 09:48:30 shirok Exp $
+ *  $Id: vport.c,v 1.13 2005/10/28 02:20:26 shirok Exp $
  */
 
 #include "gauche/vport.h"
@@ -162,16 +162,17 @@ static int vport_getz(char *buf, int buflen, ScmPort *p)
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->gets_proc)) {
-        int size;
+        u_int size;
+        const char *start;
         ScmObj s = Scm_Apply(data->gets_proc, SCM_LIST1(SCM_MAKE_INT(buflen)));
         if (!SCM_STRINGP(s)) return EOF;
-        size = SCM_STRING_SIZE(s);
+        start = Scm_GetStringContent(SCM_STRING(s), &size, NULL, NULL);
         if (size > buflen) {
             /* NB: should raise an exception? */
-            memcpy(buf, SCM_STRING_START(s), buflen);
+            memcpy(buf, start, buflen);
             return buflen;
         } else {
-            memcpy(buf, SCM_STRING_START(s), size);
+            memcpy(buf, start, size);
             return size;
         }
     } else {
@@ -283,20 +284,21 @@ static void vport_putz(const char *buf, int size, ScmPort *p)
 static void vport_puts(ScmString *s, ScmPort *p)
 {
     vport *data = (vport*)p->src.vt.data;
+    const ScmStringBody *b = SCM_STRING_BODY(s);
     SCM_ASSERT(data != NULL);
 
     if (!SCM_FALSEP(data->puts_proc)) {
         Scm_Apply(data->puts_proc, SCM_LIST1(SCM_OBJ(s)));
-    } else if (SCM_STRING_INCOMPLETE_P(s) 
+    } else if (SCM_STRING_BODY_INCOMPLETE_P(b) 
                || (SCM_FALSEP(data->putc_proc)
                    && !SCM_FALSEP(data->putb_proc))) {
         /* we perform binary output */
-        vport_putz(SCM_STRING_START(s), SCM_STRING_SIZE(s), p);
+        vport_putz(SCM_STRING_BODY_START(b), SCM_STRING_BODY_SIZE(b), p);
     } else if (!SCM_FALSEP(data->putc_proc)) {
         ScmChar c;
         int i;
-        const char *cp = SCM_STRING_START(s);
-        for (i=0; i < SCM_STRING_LENGTH(s); i++) {
+        const char *cp = SCM_STRING_BODY_START(b);
+        for (i=0; i < SCM_STRING_BODY_LENGTH(b); i++) {
             SCM_CHAR_GET(cp, c);
             cp += SCM_CHAR_NFOLLOWS(*cp)+1;
             Scm_Apply(data->putc_proc, SCM_LIST1(SCM_MAKE_CHAR(c)));
@@ -485,6 +487,8 @@ SCM_DEFINE_BASE_CLASS(Scm_BufferedOutputPortClass, ScmPort,
                       vport_print, NULL, NULL,
                       bport_allocate, vport_cpa);
 
+static ScmObj key_bufsize = SCM_FALSE; /* :buffer-size */
+
 /*
  * Scheme handlers.  They are visible from Scheme as instance slots.
  */
@@ -610,6 +614,8 @@ static ScmObj bport_allocate(ScmClass *klass, ScmObj initargs)
     bport *data = SCM_NEW(bport);
     ScmPortBuffer buf;
     int dir;
+    int bufsize = Scm_GetInteger(Scm_GetKeyword(key_bufsize, initargs,
+                                                SCM_MAKE_INT(0)));
 
     data->fill_proc  = SCM_FALSE;
     data->flush_proc = SCM_FALSE;
@@ -618,10 +624,16 @@ static ScmObj bport_allocate(ScmClass *klass, ScmObj initargs)
     data->filenum_proc = SCM_FALSE;
     data->seek_proc  = SCM_FALSE;
 
-    buf.buffer  = NULL;
+    if (bufsize > 0) {
+        buf.buffer = SCM_NEW_ATOMIC2(char*, bufsize);
+        buf.size = bufsize;
+    } else {
+        buf.buffer = NULL;
+        buf.size = 0;
+    }
+    
     buf.current = NULL;
     buf.end     = NULL;
-    buf.size    = 0;
     buf.mode    = SCM_PORT_BUFFER_FULL;
     buf.filler  = bport_fill;
     buf.flusher = bport_flush;
@@ -694,7 +706,8 @@ void Scm_Init_vport(void)
 {
     ScmModule *mod;
     SCM_INIT_EXTENSION(vport);
-    mod = SCM_MODULE(SCM_FIND_MODULE("gauche.vport", TRUE));
+    mod = SCM_FIND_MODULE("gauche.vport", SCM_FIND_MODULE_CREATE);
+    
     Scm_InitStaticClass(&Scm_VirtualInputPortClass,
                         "<virtual-input-port>", mod, viport_slots, 0);
     Scm_InitStaticClass(&Scm_VirtualOutputPortClass,
@@ -703,6 +716,9 @@ void Scm_Init_vport(void)
                         "<buffered-input-port>", mod, biport_slots, 0);
     Scm_InitStaticClass(&Scm_BufferedOutputPortClass,
                         "<buffered-output-port>", mod, boport_slots, 0);
+
+    key_bufsize = SCM_MAKE_KEYWORD("buffer-size");
+
     Scm_Init_vportlib(mod);
 }
 
