@@ -31,7 +31,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
-;;;  $Id: dbi.scm,v 1.29 2005/10/06 13:49:13 shirok Exp $
+;;;  $Id: dbi.scm,v 1.32 2006/03/09 21:53:06 shirok Exp $
 ;;;
 
 ;;; *EXPERIMENTAL*
@@ -255,21 +255,25 @@
       (call-with-output-string
         (lambda (p)
           (with-port-locking p
-            (cut generate-sql/parameters conn tokens args p)))))))
+            (cut generate-sql/parameters conn tokens args p sql)))))))
 
-(define (generate-sql/parameters conn tokens args p)
+(define (generate-sql/parameters conn tokens args p sql)
   (let loop ((tokens tokens)
              (args   args)
              (delim  #t))
     (unless (null? tokens)
       (match (car tokens)
+        ((? char? x)
+         (display x p)
+         (loop (cdr tokens) args #t))
         ((? symbol? x)
          (unless delim (write-char #\space p))
          (display x p)
          (loop (cdr tokens) args #f))
-        ((? char? x)
+        ((? string? x)
+         (unless delim (write-char #\space p))
          (display x p)
-         (loop (cdr tokens) args #t))
+         (loop (cdr tokens) args #f))
         (('delimited x)
          (unless delim (write-char #\space p))
          (format p "\"~a\"" (regexp-replace-all #/\"/ x "\"\""))
@@ -336,6 +340,6 @@
 (define-method dbi-make-query ((c <dbi-connection>) . _)
   (make <dbi-query> :connection c))
 (define-method dbi-execute-query ((q <dbi-query>) (s <string>))
-  (dbi-do (ref q 'connection) s))
+  (dbi-do (ref q 'connection) s '() :pass-through #t))
 
 (provide "dbi")
