@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;  
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: objlib.scm,v 1.3 2005/08/25 08:28:20 shirok Exp $
+;;;  $Id: objlib.scm,v 1.5 2006/03/18 04:46:03 shirok Exp $
 ;;;
 
 ;; This module is not meant to be `use'd.   It is just to hide
@@ -536,7 +536,10 @@
 (define-method x->number  ((obj <char>)) (char->integer obj))
 (define-method x->number  ((obj <top>)) 0)
 
-;; shorthand notation
+;;----------------------------------------------------------------
+;; Generic accessor
+;;
+
 (define-method ref ((obj <top>) (slot <symbol>))
   (slot-ref obj slot))
 (define-method (setter ref) ((obj <top>) (slot <symbol>) value)
@@ -549,6 +552,35 @@
 (define-method (setter ref) ((obj <hash-table>) key value)
   (hash-table-put! obj key value))
 
+;; gauche.sequence has the generic version for <sequence>, but these
+;; shortcuts would be faster.
+(define-method ref ((obj <list>) (index <integer>))
+  (list-ref obj index))
+(define-method ref ((obj <vector>) (index <integer>))
+  (vector-ref obj index))
+(define-method ref ((obj <string>) (index <integer>))
+  (string-ref obj index))
+(define-method (setter ref) ((obj <vector>) (index <integer>) val)
+  (vector-set! obj index val))
+(define-method (setter ref) ((obj <string>) (index <integer>) val)
+  (string-set! obj index val))
+
+;; ref* - chain of refs.
+;;                                        
+;; This allows us to write
+;;   (ref (ref (ref foo 'bar) 'baz) 'bang)
+;; as
+;;   (ref* foo 'bar 'baz 'bang)
+;; Credit to Issac Trotts.
+
+;; Once we have a compiler-macro, or optimized case-lambda, we
+;; can make them more efficient by expanding ref* to refs in
+;; fixed-argument case.
+(define (ref* x y . more)
+  (if (null? more)
+    (ref x y)
+    (apply ref* (ref x y) more)))
+
 ;;----------------------------------------------------------------
 ;; Generalized application hooks
 ;;  (should this be in separate file, e.g. apply.scm?)
@@ -560,11 +592,13 @@
 (define-method object-apply ((self <regmatch>) (i <integer>))
   (rxmatch-substring self i))
 (define-method object-apply ((self <regmatch>) (s <symbol>))
-  (object-apply self s 0))
-(define-method object-apply ((self <regmatch>) (s <symbol>) (i <integer>))
   (case s
-    ((before) (rxmatch-before self i))
-    ((after)  (rxmatch-after self i))
+    ((before after) (object-apply self s 0))
+    (else (rxmatch-substring self s))))
+(define-method object-apply ((self <regmatch>) (s <symbol>) group)
+  (case s
+    ((before) (rxmatch-before self group))
+    ((after)  (rxmatch-after self group))
     (else
      (errorf "bad symbol argument to ~s: ~s: must be either 'before or 'after"
              self s))))
@@ -599,6 +633,6 @@
                 slot-definition-allocation slot-definition-getter
                 slot-definition-setter slot-definition-accessor
                 class-slot-definition class-slot-accessor
-                x->string x->integer x->number ref |setter of ref|)
+                x->string x->integer x->number ref ref* |setter of ref|)
 
 (provide "gauche/object")
