@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: signal.c,v 1.33 2005/06/30 22:48:31 shirok Exp $
+ *  $Id: signal.c,v 1.39 2006/03/25 15:16:53 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -261,9 +261,15 @@ static ScmObj default_sighandler(ScmObj *args, int nargs, void *data)
         }
     }
     if (name) {
-        Scm_Error("unhandled signal %d (%s)", signum, name);
+        Scm_RaiseCondition(SCM_OBJ(SCM_CLASS_UNHANDLED_SIGNAL_ERROR),
+                           "signal", SCM_MAKE_INT(signum),
+                           SCM_RAISE_CONDITION_MESSAGE,
+                           "unhandled signal %d (%s)", signum, name);
     } else {
-        Scm_Error("unhandled signal %d (unknown signal)", signum);
+        Scm_RaiseCondition(SCM_OBJ(SCM_CLASS_UNHANDLED_SIGNAL_ERROR),
+                           "signal", SCM_MAKE_INT(signum),
+                           SCM_RAISE_CONDITION_MESSAGE,
+                           "unhandled signal %d (unknown signal)", signum);
     }
     return SCM_UNDEFINED;       /* dummy */
 }
@@ -395,8 +401,16 @@ static void sig_handle(int signum)
     if (vm == NULL) return;
     
     if (vm->sigq.overflow) {
-        /* TODO: what should we do? */
-        Scm_Warn("Signal queue overflow\n");
+        /* TODO: we should have better fallback strategy. */
+        struct sigdesc *desc = sigDesc;
+        const char *signame = "unknown signal";
+        for (; desc->name; desc++) {
+            if (desc->num == signum) {
+                signame = desc->name;
+                break;
+            }
+        }
+        Scm_Warn("Signal queue overflow by %s(%d)\n", signame, signum);
         return;
     }
     vm->sigq.queue[vm->sigq.tail++] = signum;
@@ -634,15 +648,43 @@ void Scm_SetMasterSigmask(sigset_t *set)
 ScmObj Scm_SysSigmask(int how, ScmSysSigset *newmask)
 {
     ScmSysSigset *oldmask = make_sigset();
-    if (how != SIG_SETMASK && how != SIG_BLOCK && how != SIG_UNBLOCK) {
-        Scm_Error("bad 'how' argument for signal mask action: %d", how);
+    sigset_t *newset = NULL;
+
+    if (newmask) {
+        newset = &(newmask->set);
+        if (how != SIG_SETMASK && how != SIG_BLOCK && how != SIG_UNBLOCK) {
+            Scm_Error("bad 'how' argument for signal mask action: %d", how);
+        }
     }
-    if (SIGPROCMASK(how, &(newmask->set), &(oldmask->set)) != 0) {
+    if (SIGPROCMASK(how, newset, &(oldmask->set)) != 0) {
         Scm_Error("sigprocmask failed");
     }
     return SCM_OBJ(oldmask);
 }
 
+/*
+ * Reset signal handlers except the masked ones.
+ * This is called just before we change the signal mask and call exec(2),
+ * so that we can avoid the hazard that the signal handler is called
+ * between sigsetmask and exec.
+ */
+void Scm_ResetSignalHandlers(sigset_t *mask)
+{
+    struct sigdesc *desc = sigDesc;
+    struct sigaction act;
+    int signum;
+
+    for (; desc->name; desc++) {
+        if (!sigismember(&sigHandlers.masterSigset, desc->num)
+            && (!mask || !sigismember(mask, desc->num))) {
+            act.sa_flags = 0;
+            act.sa_handler = SIG_IGN;
+            // NB: we tolerate failure of this
+            sigaction(desc->num, &act, NULL);
+        }
+    }
+}
+
 /*
  * sigsuspend
  */
