@@ -1,7 +1,7 @@
 /*
  * system.c - system interface
  *
- *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: system.c,v 1.71 2005/10/28 02:53:10 shirok Exp $
+ *  $Id: system.c,v 1.78 2006/03/25 14:15:21 shirok Exp $
  */
 
 #include <stdio.h>
@@ -183,7 +183,9 @@ ScmObj Scm_ReadDirectory(ScmString *pathname)
 }
 
 /* Glob()function. */
-/* TODO: allow to take optional flags */
+/* NB: This will soon go away.  sys-glob will be implemented in Scheme
+   on top of sys-readdir and some text manipulation, so that we'll be
+   free from system-dependent gotchas. */
 ScmObj Scm_GlobDirectory(ScmString *pattern)
 {
 #if defined(HAVE_GLOB_H)
@@ -509,12 +511,26 @@ ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
     return Scm_DStringGet(&buf, 0);        
 }
 
+/* Basename and dirname.
+   On Win32, we need to treat drive names specially, e.g.:
+   (sys-dirname "C:/a") == (sys-dirname "C:/") == (sys-dirname "C:") == "C:\\"
+   (sys-basename "C:/") == (sys-basename "C:) == "" 
+*/
+
 ScmObj Scm_BaseName(ScmString *filename)
 {
     u_int size;
     const char *path = Scm_GetStringContent(filename, &size, NULL, NULL);
     const char *endp, *last;
 
+#if defined(__MINGW32__)
+    /* Ignore drive letter, for it can never be a part of basename. */
+    if (size >= 2 && path[1] == ':' && isalpha(path[0])) {
+        path += 2;
+        size -= 2;
+    }
+#endif /* __MINGW32__ */
+
     if (size == 0) return SCM_MAKE_STR("");
     endp = truncate_trailing_separators(path, path+size);
     last = get_last_separator(path, endp);
@@ -530,18 +546,44 @@ ScmObj Scm_DirName(ScmString *filename)
     u_int size;
     const char *path = Scm_GetStringContent(filename, &size, NULL, NULL);
     const char *endp, *last;
-
-    if (size == 0) return SCM_MAKE_STR(".");
-    endp = truncate_trailing_separators(path, path+size);
-    if (endp == path) return SCM_MAKE_STR(ROOTDIR);
-    last = get_last_separator(path, endp);
-    if (last == NULL) {
-        return SCM_MAKE_STR(".");
+#if defined(__MINGW32__)
+    int drive_letter = -1;
+    if (size >= 2 && path[1] == ':' && isalpha(path[0])) {
+        drive_letter = path[0];
+        path += 2;
+        size -= 2;
     }
+#endif /* __MINGW32__ */
+
+    if (size == 0) { path = NULL; goto finale; }
+    endp = truncate_trailing_separators(path, path+size);
+    if (endp == path) { path = ROOTDIR, size = 1; goto finale; }
+    last = get_last_separator(path, endp);
+    if (last == NULL) { path = ".", size = 1; goto finale; }
 
     /* we have "something/", and 'last' points to the last separator. */
     last = truncate_trailing_separators(path, last);
-    return Scm_MakeString(path, last-path, -1, 0);
+    if (last == path) {
+        path = ROOTDIR, size = 1;
+    } else {
+        size = last - path;
+    }
+ finale:
+#if defined(__MINGW32__)
+    if (drive_letter > 0) {
+        ScmObj z;
+        char p[3] = "x:";
+        p[0] = (char)drive_letter;
+        z = Scm_MakeString(p, 2, 2, SCM_MAKSTR_COPYING);
+        if (path) {
+            return Scm_StringAppendC(SCM_STRING(z), path, size, -1);
+        } else {
+            return Scm_StringAppendC(SCM_STRING(z), ROOTDIR, 1, -1);
+        }
+    }
+#endif /* __MINGW32__ */
+    if (path) return Scm_MakeString(path, size, -1, 0);
+    else      return Scm_MakeString(".", 1, 1, 0);
 }
 
 #undef ROOTDIR
@@ -752,17 +794,33 @@ ScmObj Scm_MakeTime(ScmObj type, long sec, long nsec)
     return SCM_OBJ(t);
 }
 
-ScmObj Scm_CurrentTime(void)
+/* Abstract gettimeofday() */
+void Scm_GetTimeOfDay(u_long *sec, u_long *usec)
 {
-#ifdef HAVE_GETTIMEOFDAY
+#if defined(HAVE_GETTIMEOFDAY)
     struct timeval tv;
     int r;
     SCM_SYSCALL(r, gettimeofday(&tv, NULL));
     if (r < 0) Scm_SysError("gettimeofday failed");
-    return Scm_MakeTime(SCM_SYM_TIME_UTC, (long)tv.tv_sec, (long)tv.tv_usec*1000);
-#else  /* !HAVE_GETTIMEOFDAY */
-    return Scm_MakeTime(SCM_SYM_TIME_UTC, (long)time(NULL), 0);
-#endif /* !HAVE_GETTIMEOFDAY */
+    *sec = (u_long)tv.tv_sec;
+    *usec = (u_long)tv.tv_usec;
+#elif defined(__MINGW32__)
+    FILETIME ft;
+    GetSystemTimeAsFileTime(&ft);
+    SCM_FILETIME_TO_UNIXTIME(ft, *sec, *usec);
+#else  /* !HAVE_GETTIMEOFDAY && !__MINGW32 */
+    /* Last resort */
+    *sec = (u_long)time(NULL);
+    *usec = 0;
+#endif /* !HAVE_GETTIMEOFDAY && !__MINGW32 */
+}
+
+
+ScmObj Scm_CurrentTime(void)
+{
+    u_long sec, usec;
+    Scm_GetTimeOfDay(&sec, &usec);
+    return Scm_MakeTime(SCM_SYM_TIME_UTC, sec, usec*1000);
 }
 
 ScmObj Scm_IntSecondsToTime(long sec)
@@ -987,8 +1045,6 @@ SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_SysGroupClass, grp_print);
 
 static ScmObj make_group(struct group *g)
 {
-    ScmObj head = SCM_NIL, tail = SCM_NIL, p;
-    char **memp;
     ScmSysGroup *sg = SCM_NEW(ScmSysGroup);
     SCM_SET_CLASS(sg, SCM_CLASS_SYS_GROUP);
     
@@ -999,11 +1055,8 @@ static ScmObj make_group(struct group *g)
     sg->passwd = SCM_FALSE;
 #endif
     sg->gid = Scm_MakeInteger(g->gr_gid);
-    for (memp = g->gr_mem; *memp; memp++) {
-        p = SCM_MAKE_STR_COPYING(*memp);
-        SCM_APPEND1(head, tail, p);
-    }
-    sg->mem = head;
+    sg->mem = Scm_CStringArrayToList((const char**)g->gr_mem, -1,
+                                     SCM_MAKSTR_COPYING);
     return SCM_OBJ(sg);
 }
 
@@ -1173,42 +1226,97 @@ int Scm_IsSugid(void)
  *   reasonable way.  For now, iomap is ignored.
  */
 
-ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap, int forkp)
+ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
+                   ScmSysSigset *mask, int flags)
 {
-    int argc = Scm_Length(args), i, j, maxfd, iollen;
-    int *tofd = NULL, *fromfd = NULL, *tmpfd = NULL;
+    int argc = Scm_Length(args), i, *fds;
     char **argv;
     const char *program;
-    ScmObj ap, iop;
+    ScmObj ap;
     pid_t pid = 0;
+    int forkp = flags & SCM_EXEC_WITH_FORK;
 
     if (argc < 1) {
         Scm_Error("argument list must have at least one element: %S", args);
     }
 
     /* make a C array of C strings */    
-    argv = SCM_NEW_ARRAY(char *, argc+1);
-    for (i=0, ap = args; i<argc; i++, ap = SCM_CDR(ap)) {
-        if (!SCM_STRINGP(SCM_CAR(ap)))
-            Scm_Error("bad argument (string required): %S", SCM_CAR(ap));
-        argv[i] = Scm_GetString(SCM_STRING(SCM_CAR(ap)));
-    }
-    argv[i] = NULL;
+    argv = Scm_ListToCStringArray(args, TRUE, NULL);
     program = Scm_GetStringConst(file);
 
 #ifndef __MINGW32__
     /* setting up iomap table */
-    iollen = Scm_Length(iomap);
+    fds = Scm_SysPrepareFdMap(iomap);
+    
+    /* When requested, call fork() here. */
+    if (forkp) {
+        SCM_SYSCALL(pid, fork());
+        if (pid < 0) Scm_SysError("fork failed");
+    }
+
+    /* Now we swap file descriptors and exec(). */
+    if (!forkp || pid == 0) {
+        Scm_SysSwapFds(fds);
+        if (mask) {
+            Scm_ResetSignalHandlers(&mask->set);
+            Scm_SysSigmask(SIG_SETMASK, mask);
+        }
+        execvp(program, (char *const*)argv);
+        /* here, we failed */
+        Scm_Panic("exec failed: %s: %s", program, strerror(errno));
+    }
+
+    /* We come here only when fork is requested. */
+    return Scm_MakeInteger(pid);
+#else  /* __MINGW32__ */
+    if (forkp) {
+	Scm_Error("fork() not supported on MinGW port");
+    } else {
+	execvp(program, (const char *const*)argv);
+	Scm_Panic("exec failed: %s: %s", program, strerror(errno));	
+    }
+    return SCM_FALSE; /* dummy */
+#endif /* __MINGW32__ */
+}
+
+/* Two auxiliary functions to support iomap feature.  They are exposed
+   so that the library can implement iomap feature as the same way as
+   sys-exec.
+
+   The first function, Scm_SysPrepareFdMap, walks iomap structure and
+   prepare a table of file descriptors to modify.  The second function,
+   Scm_SysSwapFds, takes the table and modifies process's file descriptors.
+
+   We need to split this feature to two function, since it is unsafe
+   to raise an error after fork() in multi-threaded environment.
+   Scm_SysPrepareFdMap may throw an error if passed iomap contains
+   invalid entries.  On the other hand, Scm_SysSwapFds just aborts if
+   things goes wrong---not only because of the MT-safety issue, but also
+   it is generally impossible to handle errors reasonably since we don't
+   even sure we have stdios.   And the client code is supposed to call  
+   fork() between these functions.
+
+   The client code should treat the returned pointer of Scm_SysPrepareFdMap
+   opaque, and pass it to Scm_SysSwapFds as is.
+*/
+int *Scm_SysPrepareFdMap(ScmObj iomap)
+{
+    int *fds = NULL;
+#ifndef __MINGW32__
     if (SCM_PAIRP(iomap)) {
+        ScmObj iop;
+        int iollen = Scm_Length(iomap), i = 0;
+        int *tofd, *fromfd;
+        
         /* check argument vailidity before duping file descriptors, so that
            we can still use Scm_Error */
         if (iollen < 0) {
             Scm_Error("proper list required for iolist, but got %S", iomap);
         }
-        tofd   = SCM_NEW_ATOMIC2(int *, iollen * sizeof(int));
-        fromfd = SCM_NEW_ATOMIC2(int *, iollen * sizeof(int));
-        tmpfd  = SCM_NEW_ATOMIC2(int *, iollen * sizeof(int));
-        i = 0;
+        fds    = SCM_NEW_ATOMIC2(int *, 2 * iollen * sizeof(int) + 1);
+        fds[0] = iollen;
+        tofd   = fds + 1;
+        fromfd = fds + 1 + iollen;
         SCM_FOR_EACH(iop, iomap) {
             ScmObj port, elt = SCM_CAR(iop);
             if (!SCM_PAIRP(elt) || !SCM_INTP(SCM_CAR(elt))
@@ -1238,57 +1346,50 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap, int forkp)
             i++;
         }
     }
+#endif /* __MINGW32__ */
+    return fds;
+}
+
+void Scm_SysSwapFds(int *fds)
+{
+#ifndef __MINGW32__
+    int *tofd, *fromfd, nfds, maxfd, i, j, fd;
     
-    /* When requested, call fork() here. */
-    if (forkp) {
-        SCM_SYSCALL(pid, fork());
-        if (pid < 0) Scm_SysError("fork failed");
+    if (fds == NULL) return;
+
+    nfds   = fds[0];
+    tofd   = fds + 1;
+    fromfd = fds + 1 + nfds;
+
+    /* TODO: use getdtablehi if available */
+    if ((maxfd = sysconf(_SC_OPEN_MAX)) < 0) {
+        Scm_Panic("failed to get OPEN_MAX value from sysconf");
     }
 
-    /* Now we swap file descriptors and exec().
-       We can't throw an error anymore! */
-    if (!forkp || pid == 0) {
-        /* TODO: use getdtablehi if available */
-        if ((maxfd = sysconf(_SC_OPEN_MAX)) < 0) {
-            Scm_Panic("failed to get OPEN_MAX value from sysconf");
-        }
-
-        for (i=0; i<iollen; i++) {
-            if (tofd[i] == fromfd[i]) continue;
-            for (j=i+1; j<iollen; j++) {
-                if (tofd[i] == fromfd[j]) {
-                    int tmp = dup(tofd[i]);
-                    if (tmp < 0) Scm_Panic("dup failed: %s", strerror(errno));
-                    fromfd[j] = tmp;
-                }
+    /* Dup fromfd to the corresponding tofd.  We need to be careful
+       not to override the destination fd if it will be used. */
+    for (i=0; i<nfds; i++) {
+        if (tofd[i] == fromfd[i]) continue;
+        for (j=i+1; j<nfds; j++) {
+            if (tofd[i] == fromfd[j]) {
+                int tmp = dup(tofd[i]);
+                if (tmp < 0) Scm_Panic("dup failed: %s", strerror(errno));
+                fromfd[j] = tmp;
             }
-            if (dup2(fromfd[i], tofd[i]) < 0)
-                Scm_Panic("dup2 failed: %s", strerror(errno));
         }
-        for (i=0; i<maxfd; i++) {
-            for (j=0; j<iollen; j++) {
-                if (i == tofd[j]) break;
-            }
-            if (j == iollen) close(i);
-        }
-        execvp(program, (char *const*)argv);
-        /* here, we failed */
-        Scm_Panic("exec failed: %s: %s", program, strerror(errno));
+        if (dup2(fromfd[i], tofd[i]) < 0)
+            Scm_Panic("dup2 failed: %s", strerror(errno));
     }
-
-    /* We come here only when fork is requested. */
-    return Scm_MakeInteger(pid);
-#else  /* __MINGW32__ */
-    if (forkp) {
-	Scm_Error("fork() not supported on MinGW port");
-    } else {
-	execvp(program, (const char *const*)argv);
-	Scm_Panic("exec failed: %s: %s", program, strerror(errno));	
+    
+    /* Close unused fds */
+    for (fd=0; fd<maxfd; fd++) {
+        for (j=0; j<nfds; j++) if (fd == tofd[j]) break;
+        if (j == nfds) close(fd);
     }
-    return SCM_FALSE; /* dummy */
 #endif /* __MINGW32__ */
 }
 
+
 /*===============================================================
  * select
  */
