@@ -9,6 +9,8 @@
 
 (test-start "system")
 
+(define *win32* (string-suffix? "-mingw32" (gauche-architecture)))
+
 ;;-------------------------------------------------------------------
 (test-section "system")
 ;; test this first, so that we can use system commands to verify our results.
@@ -33,7 +35,8 @@
 (define (get-pwd-via-pwd)
   ;; use pwd command to get pwd.  avoid using shell's built-in pwd,
   ;; for it may be confused by symlinks.
-  (cond ((sys-access "/bin/pwd" |X_OK|) (get-command-output "/bin/pwd"))
+  (cond (*win32* (get-command-output "cd"))
+	((sys-access "/bin/pwd" |X_OK|) (get-command-output "/bin/pwd"))
         ((sys-access "/usr/bin/pwd" |X_OK|) (get-command-output "/usr/bin/pwd"))
         ((sys-access "/sbin/pwd" |X_OK|) (get-command-output "/sbin/pwd"))
         (else (get-command-output "pwd"))))
@@ -45,13 +48,10 @@
 ;;-------------------------------------------------------------------
 (test-section "environment")
 
-(test* "getenv" (get-command-output "echo $PATH")
-       (sys-getenv "PATH"))
-
 (test* "getenv"
-       (let ((x (get-command-output "echo $NoSucHEnvIRoNmenT")))
-         (if (string-null? x) #f x))
-       (sys-getenv "NoSucHEniIRoNmenT"))
+       (string-trim-both
+	(get-command-output (if *win32* "echo %PATH%" "echo $PATH")))
+       (sys-getenv "PATH"))
 
 (test* "getcwd" (get-pwd-via-pwd)
        (sys-getcwd))
@@ -74,18 +74,40 @@
 (test* "dirname"  (n "/") (sys-dirname "/"))
 (test* "basename" "" (sys-basename "//"))
 (test* "dirname"  (n "/") (sys-dirname "//"))
+(test* "basename" "abc"   (sys-basename "/abc"))
+(test* "dirname"  (n "/") (sys-dirname  "/abc"))
+(test* "basename" "abc"   (sys-basename "//abc"))
+(test* "dirname"  (n "/") (sys-dirname  "//abc"))
 
 (test* "basename" ".." (sys-basename "../"))
 (test* "dirname"  "." (sys-dirname "../"))
 (test* "basename" ".." (sys-basename "../.."))
 (test* "dirname"  ".." (sys-dirname "../.."))
 
+(when *win32*
+  ;; test with a drive letter
+  (test* "dirname"  "d:\\" (sys-dirname  "d:"))
+  (test* "basename" ""     (sys-basename "d:"))
+  (test* "dirname"  "d:\\" (sys-dirname  "d:/"))
+  (test* "basename" ""     (sys-basename "d:/"))
+  (test* "dirname"  "d:\\" (sys-dirname  "d:/z"))
+  (test* "basename" "z"    (sys-basename "d:/z"))
+  (test* "dirname"  "d:/z" (sys-dirname  "d:/z/y"))
+  (test* "basename" "y"    (sys-basename "d:/z/y"))
+  (test* "dirname"  "d:."  (sys-dirname  "d:z"))
+  (test* "basename" "z"    (sys-basename "d:z"))
+  (test* "dirname"  "d:z"  (sys-dirname  "d:z/y"))
+  (test* "basename" "y"    (sys-basename "d:z/y"))
+  )
+
 (test* "normalize" (n (string-append (get-pwd-via-pwd) "/."))
        (sys-normalize-pathname "." :absolute #t))
 (test* "normalize" (n (string-append (get-pwd-via-pwd) "/"))
        (sys-normalize-pathname "" :absolute #t))
-(test* "normalize" (n (string-append (get-command-output "echo $HOME") "/abc"))
-       (sys-normalize-pathname "~/abc" :expand #t))
+(unless *win32*
+  (test* "normalize"
+         (n (string-append (get-command-output "echo $HOME") "/abc"))
+         (sys-normalize-pathname "~/abc" :expand #t)))
 (test* "normalize" (n "/a/b/c/d/e")
        (sys-normalize-pathname "/a/b//.///c//d/./e"
                                :canonicalize #t))
@@ -311,23 +333,32 @@
                (close-input-port in) (close-output-port out)
                (list f1 f2 f3))))))
 
+;; Kludge: MinGW32 seems not to support :none, :line buffering,
+;; so we flush and close the output pipe before reading from it.
+
 (test* "pipe and read-block(none)" 2
        (receive (in out) (sys-pipe :buffering :none)
          (display "ab" out)
+         (when *win32* (close-output-port out))
          (let1 r (string-size (read-block 1000 in))
-           (close-input-port in) (close-output-port out)
+           (close-input-port in)
+           (unless *win32* (close-output-port out))
            r)))
 
 (test* "pipe and read-block(line)" 2
        (receive (in out) (sys-pipe :buffering :line)
          (display "a\n" out)
+         (when *win32* (close-output-port out))
          (let1 r (string-size (read-block 1000 in))
-           (close-input-port in) (close-output-port out)
+           (close-input-port in)
+           (unless *win32* (close-output-port out))
            r)))
 
 ;;-------------------------------------------------------------------
 (test-section "fork&exec")
 
+(unless *win32*  ;; win32 doesn't support fork at all.
+
 (test* "fork & wait" #t
        (let ((pid (sys-fork)))
          (if (= pid 0)
@@ -399,6 +430,25 @@
                        (loop (- toread r) (+ nread r)))))
                ))))
 
+(test* "fork, exec and signal mask" #t
+       (let ((nmask (make <sys-sigset>))
+             (cmask (make <sys-sigset>)))
+         (sys-sigset-fill! nmask)
+         (let ((omask (sys-sigmask SIG_SETMASK nmask))
+               (zero  (open-input-file "/dev/zero")))
+           (receive (in out) (sys-pipe :buffering :none)
+             (let1 pid
+                 (sys-fork-and-exec "cat" '("cat")
+                                    :iomap `((0 . ,zero) (1 . ,out))
+                                    :sigmask cmask)
+               (read-byte in) ;; make sure 'cat' is started
+               (sys-kill pid SIGINT)
+               (sys-sigmask SIG_SETMASK omask)
+               (sys-waitpid pid)
+               #t)))))
+
+) ;; unless *win32*
+
 ;;-------------------------------------------------------------------
 (test-section "select")
 
@@ -460,6 +510,8 @@
 ;;-------------------------------------------------------------------
 (test-section "signal handling")
 
+(unless *win32*
+
 (test* "sigalrm1" SIGALRM
        (call/cc
         (lambda (k)
@@ -577,6 +629,7 @@
                     ;;  (unless chld (loop)))
                     sig)))))))
 
+) ;; unless *win32*
 
 (test-end)
 
